#include "mbed.h"
#include "math.h"
Serial pc(USBTX, USBRX);
PwmOut roueG_sens1(D10);      //Pour ces 4 lignes, il faut déterminer les broches qui sont reliées aux deux ponts en H
PwmOut roueG_sens2(D11);      //afin de pouvoir faire tourner chacune des roues dans les deux sens.
PwmOut roueD_sens1(D8);
PwmOut roueD_sens2(D9);
AnalogIn signal(A0);          //Récupération du tableau de commande
Ticker ticker;
void commande(void);
double meas;                  //Définition des variables utiles pour le programme.
float angle;
float distance;
float ecart_roue = 0.5;       //Valeur de la distance entre les roues (en m).
float rayon_roue = 0.2;       //Valeur du rayon d'une roue (en m).
float rotation_roue = 1;      //Vitesse de rotation des roues (en m). En réalité, celle-ci est reliée au rapport cyclique du moteur.
float rapport_cyclique;       //Pour déterminer le rapport cyclique nécessaire, il faudrait faire
                              //des tests directement avec les moteurs.


int main() {
  meas=signal.read();
  roueG_sens1.period_ms(0);       //On intialise les vitesses des moteurs à 0 pour être
  roueD_sens1.period_ms(0);       //sûr d'être à l'arrêt au lancement du programme.
  roueG_sens1.write(0);
  roueD_sens1.write(0);
  roueG_sens2.period_ms(0);
  roueD_sens2.period_ms(0);
  roueG_sens2.write(0);
  roueD_sens2.write(0);
  i = 0;                          //Indice permettant de lire le tableau
  n = length(signal);
  ticker.attach(&commande, 0.01); //Appel du ticker et définition

  while(1){
  }

}

void commande(){
  global signal;
  if (i >= n){
    pc.printf("En attente de nouvelles instructions.");
    i = 0;                                              //On réinitialise l'indicage lorsqu'on est arrivé au bout du tableau
    signal = [];                                        //On vide le tableau afin de ne pas effectuer les instructions une deuxième fois.
  }
  else{
    if (i % 2){                                          //On regarde si c'est un angle ou une distance
      angle = signal[i];                                 //Lecture de la valeur de l'angle
      if (angle<0){
        distance = -angle*M_PI/180*ecart_roue;           //Calcul de la distance à parcourir avec la roue droite
        roueG_sens1.period_ms(0);
        roueD_sens1.period_ms(10);
        roueG_sens1.write(0);
        roueD_sens1.write(rapport_cyclique);
        delay(1000*distance/(rayon_roue*rotation_roue)); //On suppose que le delay est en ms (à vérfier)
        routeD_sens1.write(0);
        i++;
      }
      else{
        distance = angle*M_PI/180*ecart_roue;           //Calcul de la distance à parcourir avec la roue gauche
        roueG_sens1.period_ms(10);
        roueD_sens1.period_ms(0);
        roueG_sens1.write(rapport_cyclique);
        roueD_sens1.write(0);
        delay(1000*distance/(rayon_roue*rotation_roue)); //On suppose que le delay est en ms
        roueG_sens1.write(0);
        i++;
      }
    }
    else{
      distance = signal[i];                              //Lecture de la valeur de la distance à parcourir
      if (distance < 0){
        roueD_sens1.period_ms(0);
        roueG_sens1.period_ms(0);
        roueG_sens1.write(0);
        roueD_sens1.write(0);
        roueD_sens2.period_ms(10);
        roueG_sens2.period_ms(10);
        roueG_sens2.write(rapport_cyclique);
        roueD_sens2.write(rapport_cyclique);
        delay(-1000*distance/(rayon_roue*rotation_roue)); //On suppose que le delay est en ms
        roueD_sens2.write(0);
        roueG_sens2.write(0);
        i++;
      }
      else{
        roueD_sens2.period_ms(0);
        roueG_sens2.period_ms(0);
        roueG_sens2.write(0);
        roueD_sens2.write(0);
        roueD_sens1.period_ms(10);
        roueG_sens1.period_ms(10);
        roueG_sens1.write(rapport_cyclique);
        roueD_sens1.write(rapport_cyclique);
        delay(1000*distance/(rayon_roue*rotation_roue)); //On suppose que le delay est en ms
        roueD_sens1.write(0);
        roueG_sens1.write(0);
        i++;
      }
    }
  }
}
